﻿using Caliburn.Micro;
using System.Collections.ObjectModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Windows;
using VisualComponents.Create3D;
using VisualComponents.UX.Shared;

namespace Plugin.ComboEditorTool
{
    [Export(typeof(IActionItem))]
    public class Demo : ActionItem, IHandle<ItemSelectedMessage<ISimComponent>>
    {
        private ISelectionManager _selectionManager;
     
        public IRobotController Controller { get; set; }

        private bool _canExecute;
        public new bool CanExecute
        {
            get
            {
                return _canExecute;
            }
            set
            {
                if (_canExecute != value)
                {
                    _canExecute = value;
                    NotifyOfPropertyChange(() => CanExecute);
                }
            }
        }

        public ObservableCollection<ComboItemBase> ComboItems { get; protected set; }
        [ImportingConstructor]
        public Demo([Import]ISelectionManager SelectionManage)
            : base("SelectRobotToolId", "Select Robot Tool", string.Empty, null, MenuTools.ComboEditorTool)
        {
            _selectionManager = SelectionManage;

            this.ContextFilter = Contexts.Teach;
          

            ToolTipVisibility = Visibility.Visible;

            ToolTipHeader = "Select Robot Tool Header";
            ToolTipContent = "Select Robot Tool Content";
            ToolTipFooter = "Tell me more about Select Robot Tool";

            ToolTipHeaderSeparatorVisibility = Visibility.Collapsed;
            ToolTipFooterSeparatorVisibility = Visibility.Collapsed;

            EventMng.Subscribe(this);
        }
    
        public void Handle(ItemSelectedMessage<ISimComponent> message)
        {
            ComboItems = new ObservableCollection<ComboItemBase>();
            Controller = null;

            if (message.SelectedObject != null &&
                message.SelectedObject.GetRobot() != null)
            {
                Controller = message.SelectedObject.GetRobot().RobotController;
                if (Controller != null)
                {
                    CanExecute = true;
                    foreach (IRobotFrame baseFrame in Controller.Tools)
                    {
                        ComboItemBase item = new ComboItemBase()
                        {
                            Parent = this,
                            Content = baseFrame.Name,
                            Value = baseFrame
                        };

                        ComboItems.Add(item);
                    }
                }
            }
            if (message.SelectedObject != null &&
                message.SelectedObject.GetRobot() == null)
            {
                CanExecute = false;
            }


            NotifyOfPropertyChange(() => ComboItems);

            ComboItemBase currentSelection = null;
            if (Controller != null && Controller.CurrentTool != null)
            {
                currentSelection = ComboItems.Where(item => item.Value == Controller.CurrentTool).FirstOrDefault();
            }

            SelectedComboItem = currentSelection;

            EvaluateCanExecute();

        }

        public ComboItemBase SelectedComboItem
        {
            get
            {
                if (Controller != null)
                {
                    return ComboItems.Where(item => item.Value == Controller.CurrentTool).FirstOrDefault();
                }
                return null;
            }

            set
            {
                EventMng.Publish(new ActionItemExecutingMessage(this));
                if (Controller != null)
                {
                    if (value != null)
                    {
                        Controller.CurrentTool = value.Value as IRobotFrame;
                        IoC.Get<IConsoleViewModel>().AppendMessage("Selected Tool: " + Controller.CurrentTool.Name);
                    }
                    else
                    {
                        Controller.CurrentTool = null;
                    }
                }

                NotifyOfPropertyChange(() => SelectedComboItem);
                EventMng.Publish(new ActionItemExecutedMessage(this));
            }
        }
    }


    [Export(typeof(IPlugin))]
    public class RegisterPlugin : IPlugin
    {
        public void Exit()
        {

        }

        public void Initialize()
        {
            ICommandRegistry cmd = IoC.Get<ICommandRegistry>();

            var actionItem = cmd.FindItem("SelectRobotToolId");

            var item = new UXSiteSetup
            {
                UXSiteIdPath = "VcTabHome/VcRibbonShow",
                EntryId = actionItem.Id
            };
            cmd.RegisterActionItem(actionItem, item);
        }
    }
}
